/**
 * 判断是否是数组
 *
 * @param {any} array - 需要判断的对象
 */
function isArray(array) {
    return String.prototype.toString.call(array) == '[object Array]';
}

/**
 * 以某字符分隔字符串
 * eg: divideText(10000000000000, 3, ',') => 10,000,000,000,000
 *
 * @param {string/number} text - 需要被分割的文本
 * @param {number} len - 分割长度
 * @param {string} [replaceText=,] - 用来替换的字符
 * @returns {string}
 */
function divideText(text, len, replaceText) {
    if (replaceText === void 0) { replaceText = ','; }
    text = text.toString();
    var reg = new RegExp('(\\d)(?=(?:\\d{' + len + '})+$)', 'g');
    return text.replace(reg, '$1' + replaceText);
}
/**
 * 隐藏/替换字符串中间几位
 * eg: spliceText(18255558888, 3, 4, '****') // 182****8888
 *
 * @param {string/number} text - 需要被替换的文本
 * @param {number} start - start从前往后第几位
 * @param {number} last - end从后往前第几位
 * @param {string} replaceText - 用来替换的字符
 * @returns {string}
 */
function spliceText(text, start, last, replaceText) {
    text = text.toString();
    var reg = new RegExp('^(.{' + start + '})(?:\\d+)(.{' + last + '})$');
    return text.replace(reg, '$1' + replaceText + '$2');
}
/**
 * 金额每三位正数添加逗号(支持保留小数)
 * eg: toDecimal(10000000000000, 2) // 10,000,000,000.00
 *
 * @param {string/number} textt - 需要被替换的文本
 * @param {number} len - 保留小数位数
 * @returns {string}
 */
function toDecimal(text, len) {
    text = text.toString();
    if (/[^0-9.]/.test(text) || text == null || text == '')
        { return 0; }
    text = text.replace(/^(\d*)$/, '$1.');
    text = (text + '00').replace(/(\d*\.\d\d)\d*/, '$1');
    text = text.replace('.', ',');
    var re = /(\d)(\d{3},)/;
    while (re.test(text))
        { text = text.replace(re, '$1,$2'); }
    text = text.replace(/,(\d\d)$/, '.$1');
    if (len == 0) {
        // 不带小数位(默认是有小数位)
        var a = text.split('.');
        if (a[1] == '00') {
            text = a[0];
        }
    }
    return text;
}

/*eslint @typescript-eslint/no-explicit-any: ["error", { "ignoreRestArgs": false }]*/
/**
 * 根据对象数组的某字段进项排序
 *
 * @param {any[]} array - 需要排序的数组，其中每一项是对象
 * @param {string} key - 排序的字段
 * @param {boolean} [flag=false] - 是否需要倒序， 默认false正序，true则为倒序
 * @returns {string} - 排序后的数组
 */
function sortJSON(array, key, flag) {
    if (flag === void 0) { flag = false; }
    var sortBy = function (filed, flag, primer) {
        var flagIndex = flag ? -1 : 1;
        return function (a, b) {
            var a1 = +a[filed];
            var b1 = +b[filed];
            if (typeof primer != 'undefined') {
                a1 = primer("" + a1);
                b1 = primer("" + b1);
            }
            if (a1 < b1) {
                return flagIndex * -1;
            }
            if (a1 > b1) {
                return flagIndex * 1;
            }
            return 1;
        };
    };
    return array.sort(sortBy(key, flag, parseInt));
}

/**
 * 驼峰命名转换为短横线命名
 *
 * @param {string} text - 需要转换的驼峰命名的字符串
 * @returns {string}
 */
function camelToKebab(text) {
    return text.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
}
/**
 * 短横线命名转换为驼峰命名
 *
 * @param {string} text - 需要转换的短横线命名的字符串
 * @returns { string }
 */
function kebabToCamel(text) {
    return text.replace(/-([a-z])/g, function (g) {
        return g[1].toUpperCase();
    });
}

/**
 *  过滤文本中的html标签
 *
 * @param {string} str - 文本片段
 * @returns {string}
 */
function formatHTML(str) {
    str = str.replace(/<style([\s\S]*?)<\/style>/gi, '');
    str = str.replace(/<script([\s\S]*?)<\/script>/gi, '');
    str = str.replace(/<(div|p)[^>]*>/gi, '\n');
    str = str.replace(/<\/li>/gi, '\n');
    str = str.replace(/<li>/gi, ' * ');
    str = str.replace(/<\/ul>/gi, '\n');
    str = str.replace(/<\/p>/gi, '\n');
    str = str.replace(/<br\s*\/?>/gi, '\n');
    str = str.replace(/^\n*|<[^>]+>|\n*$/gi, '');
    str = str.replace(/<[^>]+>/gi, '');
    str = str.replace(/\n{2,}/g, '\n\n');
    str = str.replace(/\n+$/g, '');
    str = str.replace(/&nbsp;/gi, ' ');
    return str;
}

/**
 * 随机生成长度为len的字符串
 *
 * @param {number} len - 生成字符串的长度
 * @returns {string}
 */
function randomString(len) {
    return Math.random()
        .toString(36)
        .substring(2, len + 2);
}

/**
 * 获取高亮分词列表
 * @param {(string/Regexp)} keyword - 关键字
 * @param {string} fulltext - 整段文本
 * @returns {object[]}
 */
function getHighlightList(keyword, fulltext) {
    var reg = keyword instanceof RegExp ? keyword : new RegExp(keyword, 'ig');
    var list = [];
    var item = null;
    var prevIndex = 0;
    while ((item = reg.exec(fulltext)) !== null) {
        var curr = fulltext.substring(prevIndex, item.index);
        curr && list.push({ text: curr });
        list.push({ text: item[0], highlight: true });
        prevIndex = item.index + item[0].length;
    }
    list.push({ text: fulltext.substring(prevIndex) });
    return list;
}
/**
 * 获取高亮分词模板
 *
 * @param {(string/Regexp)} keyword - 关键字
 * @param {string} fulltext - 整段文本
 * @param {string} [tag=span] - 高亮文本标签
 * @param {string} [classname=highlight] - 高亮文本标签类名
 * @returns {string} tag标签包裹的字符串
 */
function getHighlightTemplate(keyword, fulltext, tag, classname) {
    if (tag === void 0) { tag = 'span'; }
    if (classname === void 0) { classname = 'highlight'; }
    var reg = keyword instanceof RegExp ? keyword : new RegExp(keyword, 'ig');
    var item = null;
    var prevIndex = 0;
    var str = '';
    while ((item = reg.exec(fulltext)) !== null) {
        var curr = fulltext.substring(prevIndex, item.index);
        str = curr ? "" + str + curr : str;
        str = str + "<" + tag + " class=\"" + classname + "\">" + item[0] + "</" + tag + ">";
        prevIndex = item.index + item[0].length;
    }
    str = "" + str + fulltext.substring(prevIndex);
    return str;
}
/**
 * 获取高亮分词
 * 具有tag参数，则返回该参数包裹的字符串
 * 否则返回数组对象
 *
 * @param {(string/Regexp)} keyword - 关键字
 * @param {string} fulltext - 整段文本
 * @param {string} [tag] - 高亮文本标签
 * @param {string} [classname] - 高亮文本标签类名
 * @returns {string | Object[]}
 */
function getHighlight(keyword, fulltext, tag, classname) {
    return tag ? getHighlightTemplate(keyword, fulltext, tag, classname) : getHighlightList(keyword, fulltext);
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __values(o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
}

function __read(o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
}

function __spread() {
    for (var ar = [], i = 0; i < arguments.length; i++)
        ar = ar.concat(__read(arguments[i]));
    return ar;
}

/*eslint @typescript-eslint/no-explicit-any: "off" */
/**
 * 求数组并集
 *
 * @param {Array} a
 * @param {Array} b
 */
function unionArr(a, b) {
    return __spread(new Set(__spread(a, b)));
}
/**
 * 求数组交集
 *
 * @param {Array} a
 * @param {Array} b
 */
function intersectArr(a, b) {
    return __spread(new Set(__spread(a).filter(function (x) { return b.includes(x); })));
}
/**
 * 求数组差集
 *
 * @param {Array} a
 * @param {Array} b
 */
function differenceArr(a, b) {
    return __spread(new Set(__spread(a).filter(function (x) { return !b.includes(x); })));
}

/**
 * 基本类型数组去重
 *
 * @param {Array} arr - 需要去重的数组
 */
function uniqueArr(arr) {
    return __spread(new Set(arr));
}
/**
 * 对象数组单个键值匹配去重
 *
 * @param {Array} arr - 需要去重的数组
 * @param {String} key - 单个键值
 */
function uniqueKeyInArr(arr, key) {
    var cache = {};
    return arr.reduceRight(function (item, next) {
        if (!cache[next[key]]) {
            cache[next[key]] = true;
            item.push(next);
        }
        return item;
    }, []);
}
/**
 * 对象数组完全匹配去重
 *
 * @param {Array} arr - 需要去重的数组
 */
function uniqueObjInArr(arr) {
    if (!isArray(arr))
        { return arr; }
    return Array.from(new Set(arr.map(function (item) { return JSON.stringify(item); }))).map(function (v) { return JSON.parse(v); });
}

/**
 * 规整多重数组
 *
 * @param {Array} arr - 需要规整的数组
 */
function flattenArr(arr) {
    if (!isArray(arr))
        { return arr; }
    return arr.reduce(function (done, curr) { return done.concat(flattenArr(curr)); }, []);
}

/**
 * localStorage获取值
 *
 * @param {string} key - 存储的键
 * @param {number} [day] - 有效天数
 */
function getStorage(key, day) {
    var dateStr = localStorage.getItem(key);
    if (!dateStr)
        { return ''; }
    if (!day)
        { return dateStr; }
    var obj = JSON.parse(dateStr);
    return new Date().getTime() - Number(obj.date) > 86400000 * day ? '' : obj.value;
}
/**
 * localStorage设置值
 *
 * @param {string} key - 存储的键
 * @param {any} value - 存储的值
 * @param {number} [day] - 有效天数
 */
function setStorage(key, value, day) {
    if (!day) {
        localStorage.setItem(key, value);
    }
    else {
        var params = {
            date: new Date().getTime(),
            value: value
        };
        localStorage.setItem(key, JSON.stringify(params));
    }
}
/**
 * localStorage移除键为key的存储
 *
 * @param {string} key - 存储的键
 */
function removeStorage(key) {
    localStorage.removeItem(key);
}
/**
 * localStorage清空存储
 */
function clearStorage() {
    localStorage.clear();
}

/**
 * 设置Cookie
 *
 * @param {string} key - 存储的键
 * @param {any} value - 存储的值
 * @param {number} [expiredays] - 有效天数
 * @param {string} [domain] - 共享Cookie域名
 */
function setCookie(key, value, expiredays, domain) {
    value = encodeURIComponent(value);
    var exdate = new Date();
    if (expiredays) {
        exdate.setDate(exdate.getDate() + expiredays);
    }
    var exdatestr = exdate.toUTCString();
    document.cookie = domain
        ? key + "=" + value + ";expires=" + exdatestr + ";path=/;domain=" + domain
        : key + "=" + value + ";expires=" + exdatestr + ";path=/;";
}
/**
 * 获取Cookie
 *
 * @param key
 */
function getCookie(key) {
    var reg = new RegExp("(^| )" + key + "=([^;]*)(;|$)");
    var arr = document.cookie.match(reg);
    return arr ? decodeURIComponent(arr[2]) : null;
}
/**
 * 清除Cookie
 *
 * @param {string} key - 存储的键
 * @param {string} [domain] - 共享Cookie域名
 */
function removeCookie(key, domain) {
    setCookie('', key, -1, domain);
}

/**
 *  Map转换成Object
 *
 * @param {Map} map - map对象
 * @returns {Object}
 */
function mapToObj(map) {
    var e_1, _a;
    var obj = Object.create(null);
    try {
        for (var map_1 = __values(map), map_1_1 = map_1.next(); !map_1_1.done; map_1_1 = map_1.next()) {
            var _b = __read(map_1_1.value, 2), k = _b[0], v = _b[1];
            obj[k] = v;
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (map_1_1 && !map_1_1.done && (_a = map_1.return)) { _a.call(map_1); }
        }
        finally { if (e_1) { throw e_1.error; } }
    }
    return obj;
}
/**
 *  Object转换成Map
 *
 * @param {Object} obj - map对象
 * @returns {Map}
 */
function objToMap(obj) {
    var e_2, _a;
    var map = new Map();
    try {
        for (var _b = __values(Object.keys(obj)), _c = _b.next(); !_c.done; _c = _b.next()) {
            var k = _c.value;
            map.set(k, obj[k]);
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) { _a.call(_b); }
        }
        finally { if (e_2) { throw e_2.error; } }
    }
    return map;
}

/**
 * 是否具有 className
 *
 * @param [HTMLElement] el - 元素
 * @param [String] cls - 类名
 */
function hasClass(el, cls) {
    return el.className.match(new RegExp('(\\s|^)' + cls + '(\\s|$)'));
}
/**
 * 添加 className
 *
 * @param [HTMLElement] el - 元素
 * @param [String] cls - 类名
 */
function addClass(el, cls) {
    if (hasClass(el, cls))
        { return; }
    el.className += el.className ? " " + cls : cls;
}
/**
 * 删除 className
 *
 * @param [HTMLElement] el - 元素
 * @param [String] cls - 类名
 */
function removeClass(el, cls) {
    if (!hasClass(el, cls))
        { return; }
    var reg = new RegExp('(\\s|^)' + cls + '(\\s|$)');
    el.className = el.className.replace(reg, ' ');
}
/**
 * 切换 className
 *
 * @param [HTMLElement] el - 元素
 * @param [String] cls - 类名
 */
function toggleClass(el, cls) {
    return !hasClass(el, cls) ? addClass(el, cls) : removeClass(el, cls);
}

/**
 * 获取链接中键相应的值
 *
 * @param {string} name - 键名
 * @returns {string}
 */
function getUrlParam(name) {
    var reg = new RegExp('(^|&)' + name + '=([^&]*)(&|$)', 'i');
    var r = window.location.search.substr(1).match(reg);
    return r ? decodeURIComponent(r[2]) : null;
}
/**
 * 返回链接中所有键值对象
 *  @returns {Object}
 */
function getUrlObj() {
    var qs = location.search.length > 0 ? location.search.substring(1) : '';
    var args = {};
    var items = qs.length ? qs.split('&') : [];
    var item = null;
    var name = null;
    var value = null;
    var i = 0;
    var len = items.length;
    for (i = 0; i < len; i++) {
        item = items[i].split('=');
        name = decodeURIComponent(item[0]);
        value = decodeURIComponent(item[1]);
        if (name.length) {
            args[name] = value;
        }
    }
    return args;
}
/**
 *
 * 向链接中添加键值对
 *
 * @param {string} url - 链接
 * @param {string} name - 键
 * @param {any} value - 值
 * @returns {string}
 */
function addUrlParam(url, name, value) {
    url += url.indexOf('?') == -1 ? '?' : '&';
    url += encodeURIComponent(name) + '=' + encodeURIComponent(value);
    return url;
}
/**
 * 修改链接中键对应的值
 *
 * @param {string} url - 链接
 * @param {string} name - 键
 * @param {any} value - 值
 * @returns {string}
 */
function modifyUrlParam(url, key, value) {
    var pattern = key + '=([^&]*)';
    var replaceText = key + '=' + value;
    if (url.match(pattern)) {
        var tmp = '/(' + key + '=)([^&]*)/gi';
        return url.replace(eval(tmp), replaceText);
    }
    return url.match('[?]') ? url + '&' + replaceText : url + '?' + replaceText;
}

/**
 * 判断是否是IE浏览器
 *
 * @returns {Boolean}
 */
function isIE() {
    return window['ActiveXObject'] !== 'undefined' || 'ActiveXObject' in window;
}
/**
 * 判断是否是IE11
 *
 * @returns {Boolean}
 */
function isIE11() {
    var ua = navigator.userAgent;
    return ua.indexOf('Trident') > -1 && ua.indexOf('rv:11.0') > -1;
}
/**
 * 判断是否是Edge
 *
 * @returns {Boolean}
 */
function isEdge() {
    return navigator.userAgent.indexOf('Edge') > -1;
}
/**
 * 获取IE浏览器版本
 */
function getIEVersion() {
    var match = navigator.appVersion.match(/MSIE\s+\d+.0;/);
    if (match == null)
        { return false; }
    var result = match[0].match(/\d+/);
    if (result && isArray(result)) {
        return +result[0];
    }
    return null;
}
/**
 * 判断是否是Android
 *
 * @returns {Boolean}
 */
function isAndroid() {
    return /(Android)/i.test(navigator.userAgent);
}
/**
 * 判断是否是IOS
 *
 * @returns {Boolean}
 */
function isIOS() {
    return /(iPhone|iPad|iPod|iOS)/i.test(navigator.userAgent);
}
/**
 * 判断是否是移动端
 *
 * @returns {Boolean}
 */
function isMobile() {
    return isAndroid() || isIOS();
}
/**
 * 判断是否是PC
 *
 * @returns {Boolean}
 */
function isPC() {
    return !isMobile();
}
/**
 * 判断是否是微信浏览器
 *
 * @returns {Boolean}
 */
function isWX() {
    return /micromessenger/.test(navigator.userAgent.toLowerCase());
}

/**
 *  获取事件源
 *
 * @param {Event} event - 事件源
 */
function getEvent(event) {
    if (event)
        { return event; }
    return window['event'] ? window['event'] : null;
}
/**
 * 获取事件对象
 *
 * @param {Event} event - 事件
 */
function getEventTarget(event) {
    var e = getEvent(event);
    return e.target || e.srcElement;
}
/**
 * 添加事件
 *
 * @param {Element} element - 元素
 * @param {String} type - 事件类型
 * @param {EventListener} handler - 事件
 */
function addHandler(element, type, handler) {
    if (element.addEventListener) {
        element.addEventListener(type, handler, false);
    }
    else if (element['attachEvent']) {
        element['attachEvent']("on" + type, handler);
    }
    else {
        element["on" + type] = handler;
    }
}
/**
 * 移除事件
 *
 * @param {Element} element - 元素
 * @param {String} type - 事件类型
 * @param {EventListener} handler - 事件
 */
function removeHandler(element, type, handler) {
    if (element.removeEventListener) {
        element.removeEventListener(type, handler, false);
    }
    else if (element['detachEvent']) {
        element['detachEvent']("on" + type, handler);
    }
    else {
        element["on" + type] = null;
    }
}
/**
 * 阻止默认事件
 */
function preventDefault(event) {
    var e = getEvent(event);
    if (e.preventDefault) {
        e.preventDefault();
    }
    else {
        e.returnValue = false;
    }
}
/**
 * 阻止事件冒泡
 */
function stopPropagation(event) {
    var e = getEvent(event);
    if (e.stopPropagation) {
        e.stopPropagation();
    }
    else {
        e.cancelBubble = true;
    }
}
/**
 * 获取当前位置的X轴坐标
 *
 * @param {Event} event - 事件源
 */
function getPageX(event) {
    var e = getEvent(event);
    return typeof e.pageX === 'undefined'
        ? e.clientX + (document.body.scrollLeft || document.documentElement.scrollLeft)
        : e.pageX;
}
/**
 * 获取当前位置的Y轴坐标
 *
 * @param {Event} event - 事件源
 */
function getPageY(event) {
    var e = getEvent(event);
    return typeof e.pageY === 'undefined'
        ? e.clientY + (document.body.scrollTop || document.documentElement.scrollTop)
        : e.pageX;
}
/**
 * 获取页面滚动高度
 *
 * @returns {Number}
 */
function getScrollTop() {
    return document.documentElement.scrollTop || window.pageYOffset || document.body.scrollTop;
}

/**
 * 缓存函数调用结果
 *
 * @param {function} fn - 函数
 */
function memoize(fn) {
    var cachedArg;
    var cachedResult;
    return function (arg) {
        if (cachedArg === arg)
            { return cachedResult; }
        cachedArg = arg;
        cachedResult = fn(arg);
        return cachedResult;
    };
}

export { addClass, addHandler, addUrlParam, camelToKebab, clearStorage, differenceArr, divideText, flattenArr, formatHTML, getCookie, getEvent, getEventTarget, getHighlight, getHighlightList, getHighlightTemplate, getIEVersion, getPageX, getPageY, getScrollTop, getStorage, getUrlObj, getUrlParam, hasClass, intersectArr, isAndroid, isArray, isEdge, isIE, isIE11, isIOS, isMobile, isPC, isWX, kebabToCamel, mapToObj, memoize, modifyUrlParam, objToMap, preventDefault, randomString, removeClass, removeCookie, removeHandler, removeStorage, setCookie, setStorage, sortJSON, spliceText, stopPropagation, toDecimal, toggleClass, unionArr, uniqueArr, uniqueKeyInArr, uniqueObjInArr };
